@@ -266,6 +266,8 @@ class ParsedUrl:
 
         try:
             self.path = pu.path
+            if self.path:
+                self.path = urllib.unquote(self.path)
         except Exception:
             raise InvalidBackendURL("Syntax error (path) in: %s" % url_string)
 
@@ -299,7 +301,9 @@ class ParsedUrl:
         except Exception:  # not raised in python2.7+, just returns None
             # old style rsync://host::[/]dest, are still valid, though they contain no port
             if not (self.scheme in ['rsync'] and re.search('::[^:]*$', self.url_string)):
-                raise InvalidBackendURL("Syntax error (port) in: %s A%s B%s C%s" % (url_string, (self.scheme in ['rsync']), re.search('::[^:]+$', self.netloc), self.netloc))
+                raise InvalidBackendURL("Syntax error (port) in: %s A%s B%s C%s" %
+                                        (url_string, (self.scheme in ['rsync']),
+                                         re.search('::[^:]+$', self.netloc), self.netloc))
 
         # Our URL system uses two slashes more than urlparse's does when using
         # non-netloc URLs.  And we want to make sure that if urlparse assuming
@@ -455,7 +459,7 @@ class Backend(object):
         import shlex
         from subprocess import Popen, PIPE
         args = shlex.split(commandline)
-        args[0] = self.which(args[0])
+        args[0] = util.which(args[0])
         p = Popen(args, stdout=PIPE, stderr=PIPE)
         stdout, stderr = p.communicate()
 
@@ -488,28 +492,6 @@ class Backend(object):
                                        (private, result, stdout + '\n' + stderr))
         return result, stdout, stderr
 
-    def which(self, program):
-        """
-        Return absolute path for program name.
-        Returns None if program not found.
-        """
-
-        def is_exe(fpath):
-            return os.path.isfile(fpath) and os.path.isabs(fpath) and os.access(fpath, os.X_OK)
-
-        fpath, fname = os.path.split(program)
-        if fpath:
-            if is_exe(program):
-                return program
-        else:
-            for path in os.getenv("PATH").split(os.pathsep):
-                path = path.strip('"')
-                exe_file = os.path.abspath(os.path.join(path, program))
-                if is_exe(exe_file):
-                    return exe_file
-
-        return None
-
 
 class BackendWrapper(object):
     """
@@ -605,7 +587,10 @@ class BackendWrapper(object):
 
     @retry('delete', fatal=False)
     def _do_delete_list(self, filename_list):
-        self.backend._delete_list(filename_list)
+        while filename_list:
+            sublist = filename_list[:100]
+            self.backend._delete_list(sublist)
+            filename_list = filename_list[100:]
 
     @retry('delete', fatal=False)
     def _do_delete(self, filename):
